package de.w665.sharepulse.db;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.gen.exc.ReqlOpFailedError;
import com.rethinkdb.net.Connection;
import de.w665.sharepulse.db.repo.UserRepository;
import de.w665.sharepulse.model.User;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import java.util.Optional;

@Slf4j
@Service
public class RethinkDBService {

    private final RethinkDBConfig config;
    private final RethinkDB r;
    private final Connection connection;
    private final UserRepository userRepository;

    @Value("${sharepulse.auto-reset-on-startup}")
    private boolean autoResetOnStartup;
    @Value("${sharepulse.management.user.username}")
    private String defaultUsername;
    @Value("${sharepulse.management.user.password}")
    private String defaultPassword;

    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    @Autowired
    public RethinkDBService(RethinkDBConfig config, RethinkDBConnector connector, UserRepository userRepository) {
        this.config = config;

        // mapping to private vars for easier access
        this.r = connector.getR();
        this.connection = connector.getConnection();
        this.userRepository = userRepository;
    }

    @PostConstruct
    public void initialize() {

        // rethinkdb check if database exists
        try {
            r.dbCreate(config.getDatabase()).run(connection).stream();
            log.debug("Database " + config.getDatabase() + " created");
        } catch (ReqlOpFailedError e) {
            log.debug("Database " + config.getDatabase() + " already exists. Error: " + e.getClass().getSimpleName());
        }

        // rethinkdb check if table file_uploads exists
        try {
            r.db(config.getDatabase()).tableCreate("file_uploads").run(connection).stream();
            log.debug("Table 'file_uploads' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'file_uploads' already exists.");
            if(autoResetOnStartup) {
                log.debug("Clearing content...");
                r.db(config.getDatabase()).table("file_uploads").delete().run(connection);
                log.debug("Table 'file_uploads' cleared successfully.");
            }
        }

        // rethinkdb check if table id_store exists
        try {
            r.db(config.getDatabase()).tableCreate("id_store").run(connection).stream();
            log.debug("Table 'id_store' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'id_store' already exists.");
            if(autoResetOnStartup) {
                log.debug("Clearing content...");
                r.db(config.getDatabase()).table("id_store").delete().run(connection);
                log.debug("Table 'id_store' cleared successfully.");
            }
        }

        // rethinkdb check if table expired_file_uploads exists
        try {
            r.db(config.getDatabase()).tableCreate("expired_file_uploads").run(connection).stream();
            log.debug("Table 'expired_file_uploads' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'expired_file_uploads' already exists.");
            if(autoResetOnStartup) {
                log.debug("Clearing content...");
                r.db(config.getDatabase()).table("expired_file_uploads").delete().run(connection);
                log.debug("Table 'expired_file_uploads' cleared successfully.");
            }
        }

        // rethinkdb check if table users exists
        try {
            r.db(config.getDatabase()).tableCreate("users").run(connection).stream();
            log.debug("Table 'users' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'users' already exists.");
            if(autoResetOnStartup) {
                log.debug("Clearing content...");
                r.db(config.getDatabase()).table("users").delete().run(connection);
                log.debug("Table 'users' cleared successfully.");
            }
        }

        initializeAdminUser();

        log.info("Database ready for operation!");
    }

    private void initializeAdminUser() {
        Optional<User> adminUser = userRepository.retrieveUserByUsername("admin");
        if(adminUser.isEmpty()) {
            User user = new User();
            user.setUsername(defaultUsername);
            user.setPassword(passwordEncoder.encode(defaultPassword));
            user.setRole("ADMIN");
            userRepository.insertUser(user);
            log.debug("Admin user created with default credentials. Username: admin, Password: admin");
        }
    }

    @PreDestroy
    public void close() {
        if (connection != null) {
            connection.close();
        }
    }
}
