package de.w665.sharepulse;

import de.w665.sharepulse.db.repo.UserRepository;
import de.w665.sharepulse.model.User;
import de.w665.sharepulse.service.AuthenticationService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@SpringBootTest
public class AuthenticationServiceTest {

    @MockBean
    private UserRepository userRepository;

    @Autowired
    private AuthenticationService authenticationService;

    private final String username = "testUser";
    private final String password = "testPass";

    @BeforeEach
    public void setup() {
        User mockUser = new User();
        mockUser.setUsername(username);
        mockUser.setPassword(new BCryptPasswordEncoder().encode(password));
        when(userRepository.retrieveUserByUsername(anyString())).thenReturn(Optional.of(mockUser));
    }

    @Test
    public void whenValidUsernameAndPassword_thenAuthenticateShouldReturnToken() {
        String token = authenticationService.authenticate(username, password);
        assertNotNull(token, "Token should not be null for valid credentials");
    }

    @Test
    public void whenValidToken_thenValidateTokenShouldReturnTrue() {
        String token = authenticationService.authenticate(username, password);
        assertTrue(authenticationService.validateToken(token), "Token validation should return true for a valid token");
    }

    @Test
    public void whenInvalidToken_thenValidateTokenShouldReturnFalse() {
        String invalidToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJ0ZXN0VXNlciJ9.WrongSignature";
        assertFalse(authenticationService.validateToken(invalidToken), "Token validation should return false for an invalid token");
    }

    @Test
    public void whenTokenIsExplicitlyExpired_thenValidateTokenShouldReturnFalse() throws InterruptedException {
        long testExpirationTime = 1; // 1 millisecond
        String token = authenticationService.authenticate("testUser", "testPass", testExpirationTime);
        assertNotNull(token, "Token should not be null");
        Thread.sleep(2); // Wait for 2 milliseconds to ensure the token has expired (Bad practice but easy)
        assertFalse(authenticationService.validateToken(token), "Expired token should not be valid");
    }
}
